unit Datebar;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Menus, 
  Classes, Graphics, Controls, Forms, Dialogs, 
  StdCtrls, ExtCtrls, Spin;

const
   m_DaysPerMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
   m_MonthTitles : Array[1..12] of string[20] = ('January','February','March',
                                                 'April','May','June','July',
                                                 'August', 'September',
                                                 'October', 'November',
                                                 'December');
   m_DayTitles : Array[1..7] of string[2] = ('Su','Mo','Tu','We','Th','Fr','Sa');

   TEXT_INDENT = 2;

type
  TDates = class(TStringList);
  TDays = class(TStringList);
  TCopyright = class(TStringList);
  TDateList = class(TDates);
  TDayList = class(TDays);
  TAbout = class(TCopyright)
   end;
  TDateBar = class(TPaintBox)
  private
    FSelectBrush: TBrushStyle;
    FButtonHeight: Integer;
    FAbout: TAbout;
    FMultiDaySelect: Boolean;
    FShowButtons: Boolean;
    FDateListStart: TDateList;
    FDateListEnd: TDateList;
    FDayList: TDayList;
    FDateText: String;
    FStartDateText: String;
    FEndDateText: String;
    FMonthText : String[20];
    FDateText2: String[60];
    FBackgroundColor: TColor;
    FSelectedColor: TColor;
    FLineColor: TColor;
    FShadowColor: TColor;
    FHighlightColor: TColor;
    FDateBoxColor: TColor;
    FLineSelectColor: TColor;
    FDateColor: TColor;
    FWeekendColor: TColor;
    FWeekdayColor: TColor;
    FDateTextFont: TFont;
    FDisplaySelectCaption: Boolean;
    FDateSlots: Integer;
    FStartDay: Integer;
    FSelectByDay: Boolean;
    FTitleHeight : Integer;
    FDateHeight : Integer;
    FButtonIndent: Integer;
    FSelectDateHeight : Integer;
    FCenterDay: Boolean;
    Bounding: Boolean;
    AnchorX: Integer;
    AnchorY: Integer;
    CurX: Integer;
    CurY: Integer;
    RefreshRect: TRect;
    FOnDateSelected: TNotifyEvent;
    FOnNextMonth : TNotifyEvent;
    FOnPrevMonth : TNotifyEvent;
    DateOverride: Boolean;
    procedure SetAbout(Value: TAbout);
    procedure SeTDateBarColor(Value: TColor);
    procedure SetLineColor(Value: TColor);
    procedure SetShadowColor(Value: TColor);
    procedure SetHighlightColor(Value: TColor);
    procedure SetDateBoxColor(Value: TColor);
    procedure SetLineSelectColor(Value: TColor);
    procedure SetSelectedColor(Value: TColor);
    procedure SetDateColor(Value: TColor);
    procedure SetWeekendColor(Value: TColor);
    procedure SetWeekdayColor(Value: TColor);
    procedure SetStartDay(Value: Integer);
    procedure SetDateSlots(Value: Integer);
    procedure SetDateTextFont(Value: TFont);
    procedure SetCenterDay(Value: Boolean);
    procedure SetDisplaySelectCaption(Value: Boolean);
    procedure SetDateText(Value: String);
    procedure SetStartDateText(Value: String);
    procedure SetEndDateText(Value: String);
    procedure SetDateListStart(Value: TDateList);
    procedure SetDateListEnd(Value: TDateList);
    procedure SetSelectByDay(Value: Boolean);
    procedure SetDayList(Value: TDayList);
    procedure DrawSegments;
    procedure RefreshSelect;
    procedure DrawSelect(DrawRect: TRect);
    procedure DrawSelectedDateSlots;
  protected
    function  IsLeapYear(wYear : Word): Boolean;
    function  DaysInMonth(nMonth : Integer; wYear : Word): Integer;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: integer ); override;
    procedure WMPaintItem(var Message: TWMPaint); message WM_PAINT;
    procedure DrawButtons;
    function  GetLeftButtonRect : TRect;
    function  GetRightButtonRect : TRect;
    procedure NextMonth;
    procedure PrevMonth;
    procedure RestoreMonthText;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    function  GetDaySlot(Value: Integer): String;
    function  GetSlotNumber(Value: String): Integer;
    procedure SelectByDay;
  published
    property About: TAbout read FAbout write SetAbout;
    property Color : TColor read FBackgroundColor write SeTDateBarColor;
    property LineColor : TColor read FLineColor write SetLineColor;
    property ShadowColor : TColor read FShadowColor write SetShadowColor;
    property HighlightColor : TColor read FHighlightColor write SetHighlightColor;
    property DateBoxColor: TColor read FDateBoxColor write SetDateBoxColor;
    property DisplaySelectCaption: Boolean read FDisplaySelectCaption write SetDisplaySelectCaption;
    property DateColor: TColor read FDateColor write SetDateColor;
    property WeekendColor: TColor read FWeekendColor write SetWeekendColor;
    property WeekdayColor: TColor read FWeekdayColor write SetWeekdayColor;
    property LineSelectColor: TColor read FLineSelectColor write SetLineSelectColor;
    property DateTextFont: TFont read FDateTextFont write SetDateTextFont;
    property CurrentDate: String read FDateText write SetDateText;
    property StartDate: String read FStartDateText write SetStartDateText;
    property EndDate: String read FEndDateText write SetEndDateText;
    property DateSlots: Integer read FDateSlots write SetDateSlots;
    property StartDay: Integer read FStartDay write SetStartDay;
    property CenterDay: Boolean read FCenterDay write SetCenterDay;
    property SelectedStartDate: TDateList read FDateListStart write SetDateListStart;
    property SelectedEndDate: TDateList read FDateListEnd write SetDateListEnd;
    property SelectedDay: TDayList read FDayList write SetDayList;
    property SelectedColor: TColor read FSelectedColor write SetSelectedColor;
    property OnNextMonth : TNotifyEvent read FOnNextMonth write FOnNextMonth;
    property OnPrevMonth : TNotifyEvent read FOnPrevMonth write FOnPrevMonth;
    property OnDateSelected : TNotifyEvent read FOnDateSelected write FOnDateSelected;
    property SelectBrushStyle: TBrushStyle read FSelectBrush write FSelectBrush;
    property ButtonHeight: Integer read FButtonHeight write FButtonHeight;
    property ShowButtons: Boolean read FShowButtons write FShowButtons;
    property ButtonIndent: Integer read FButtonIndent write FButtonIndent;
    property MultiDaySelect: Boolean read FMultiDaySelect write FMultiDaySelect;
    property Align;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property Left;
    property Name;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Tag;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional',[TDateBar]);
end;

{********************** Is Leap Year **********************
 **********************************************************}
function TDateBar.IsLeapYear(wYear: Word): Boolean;
begin
  Result := ( wYear mod 4 = 0 ) and
            ( ( wYear mod 100 <> 0 ) or ( wYear mod 400 = 0 ) );
end;

{********************** Days In Month *********************
 *****  This function returns the number of days in   *****
 *****  the month specified in nMonth.                *****
 **********************************************************}
function TDateBar.DaysInMonth(nMonth : Integer; wYear : Word): Integer;
begin
  Result := m_DaysPerMonth[nMonth];
  if ( nMonth = 2 ) and IsLeapYear(wYear) then Inc( Result );
end;

function PointInRect( const rectTest: TRect; X, Y: integer ): boolean;
begin
  Result := ( ( X >= rectTest.Left ) and ( X <= rectTest.Right ) and
     ( Y >= rectTest.Top ) and ( Y <= rectTest.Bottom ) );
end;

{********************* Get Left Button Rectangle ******************
 ***** Get the rectangle used for the left button.            *****
 ******************************************************************}
function TDateBar.GetLeftButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Left Button Rectangle}
   TempRect.Top := ClientRect.Top + FButtonIndent;
   TempRect.Bottom := TempRect.Top + FButtonHeight;
   TempRect.Left := FButtonIndent + TEXT_INDENT;
   TempRect.Right := TempRect.Left + FButtonHeight;

   Result := TempRect;
end;

{******************** Get Right Button Rectangle ******************
 ***** Get the rectangle used for the right button.           *****
 ******************************************************************}
function TDateBar.GetRightButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Right Button Rectangle}
   TempRect.Top := ClientRect.Top + FButtonIndent;
   TempRect.Bottom := TempRect.Top + FButtonHeight;
   TempRect.Right := Width-(FButtonIndent + TEXT_INDENT);
   TempRect.Left := TempRect.Right - FButtonHeight;

   Result := TempRect;
end;

procedure TDateBar.DrawButtons;
var
  LeftButtonRect: TRect;
  RightButtonRect : TRect;
  OldStyle : TBrushStyle;
begin
  if FShowButtons then
  with Canvas do
     begin
        LeftButtonRect := GetLeftButtonRect;
        RightButtonRect := GetRightButtonRect;

        { Select Black Pen}
        Pen.Style := psSolid;
        Pen.Width := 1;
        Pen.Color := clBlack;

        { Draw Button Outlines }
        Rectangle( LeftButtonRect.Left, LeftButtonRect.Top, LeftButtonRect.Right, LeftButtonRect.Bottom );
        Rectangle( RightButtonRect.Left, RightButtonRect.Top, RightButtonRect.Right, RightButtonRect.Bottom );

        { Create Embossed effect - Outline left & upper in white}
        Pen.Color := clWhite;
        MoveTo( LeftButtonRect.Left + 1, LeftButtonRect.Bottom - 2 );
        LineTo( LeftButtonRect.Left + 1, LeftButtonRect.Top + 1 );
        LineTo( LeftButtonRect.Right - 2, LeftButtonRect.Top + 1 );

        MoveTo( RightButtonRect.Left + 1, RightButtonRect.Bottom - 2 );
        LineTo( RightButtonRect.Left + 1, RightButtonRect.Top + 1 );
        LineTo( RightButtonRect.Right - 2, RightButtonRect.Top + 1 );

        { Create Embossed effect - Outline right & bottom in gray }
        Pen.Color := clGray;
        MoveTo( LeftButtonRect.Right -2, LeftButtonRect.Top +  1 );
        LineTo( LeftButtonRect.Right - 2, LeftButtonRect.Bottom - 2 );
        LineTo( LeftButtonRect.Left + 1, LeftButtonRect.Bottom - 2 );

        MoveTo( RightButtonRect.Right - 2, RightButtonRect.Top + 1 );
        LineTo( RightButtonRect.Right - 2, RightButtonRect.Bottom - 2 );
        LineTo( RightButtonRect.Left + 1, RightButtonRect.Bottom - 2 );

        {Draw Arrow}
        Brush.Color := clBlack;
        OldStyle :=Brush.Style;
        Brush.Style := bsSolid;
        Polygon([Point(LeftButtonRect.Right - 5,LeftButtonRect.Top + 3),
                 Point(LeftButtonRect.Right - 5,LeftButtonRect.Bottom - 4),
                 Point(LeftButtonRect.Left + 3,LeftButtonRect.Top + 7)]);

        Polygon([Point(RightButtonRect.Left + 4,RightButtonRect.Top + 3),
                 Point(RightButtonRect.Left + 4,RightButtonRect.Bottom - 4),
                 Point(RightButtonRect.Right - 4,RightButtonRect.Top + 7)]);

        Brush.Color :=clSilver;
        Brush.Style := OldStyle;
        Pen.Color := clBlack;
     end;
end;

constructor TDateBar.Create(AOwner:TComponent);
var
 Year, Month, Day: Word;
 DateVal : TDateTime;
begin
  inherited Create(AOwner);
  FMultiDaySelect := TRUE;
  Height := 70;
  Width := 434;
  FButtonHeight := 14;
  FButtonIndent := 2;
  ShowHint := TRUE;
  Hint := 'Hold the [Ctrl] key down to select multiple dates';
  FBackgroundColor := clBtnFace;
  FShadowColor := clBtnShadow;
  FSelectedColor := clHighlight;
  FHighlightColor := clBtnHighlight;
  FLineColor := clBtnText;
  FDateBoxColor := clActiveCaption;
  FLineSelectColor := clGray;
  FDateColor := clBlack;
  FWeekendColor := clBlue;
  FWeekdayColor := clBlack;
  FShowButtons := TRUE;
  FDateSlots := 31;
  FStartDay := 1;
  Font.Name := 'Small Fonts';
  Font.Size := 6;
  FSelectBrush := bsSolid;
  FDateTextFont := TFont.Create;
  FDateTextFont.Style := [fsBold];
  FDateTextFont.Color := clCaptionText;
  FDateListStart := TDateList.Create;
  FDateListEnd := TDateList.Create;
  FDayList := TDayList.Create;
  FDateText := DateToStr(Now);
  FStartDateText := DateToStr(Now);
  FEndDateText := DateToStr(Now);
  DateVal :=  StrtoDate(FDateText);
  DecodeDate(DateVal,Year, Month, Day);
  FDisplaySelectCaption := False;
{  FDateSlots := DaysInMonth(Month, Year);}
  FDateSlots := 31;
  FMonthText := m_MonthTitles[Month] + ' ' + InttoStr(Year);

  FAbout := TAbout.Create;
  FAbout.Clear;
  FAbout.Add('TDateBar - Version 1.0');
  FAbout.Add('-----------------------------');
  FAbout.Add(' ');
  FAbout.Add('Copyright  1996 by Jim Wilhelm');
end;



procedure TDateBar.WMPaintItem(var Message: TWMPaint);
var
 Sections, N, N2, X,
 Year, Month, Day, CurrDay : Word;
 NewRect: TRect;
 SavedColor: TColor;
 DateVal : TDateTime;
begin
  if FDateText <> '' then
  begin
    DateVal :=  StrtoDate(FDateText);
    DecodeDate(DateVal,Year, Month, Day);
{    FDateSlots := DaysInMonth(Month, Year);}
  end;

{  Font.Assign(Font);}
 while (((((Width-4)-FDateSlots) mod FDateSlots) > 0) and ((Width-4) > 0)) do
  begin
    Width := Width -1;
    if (Width < (12 * FDateSlots)) then
       Width := (12 * FDateSlots) + 4;
  end;
 Canvas.Font.Assign(Font);
 with Canvas do
  begin
   Pen.Color := clBlack;
   Brush.Color := FBackgroundColor;
    FillRect(ClientRect);
     with ClientRect do
      begin
       Rectangle(Left+2, Top+1, Right-1, Bottom-1);
      end;
      Sections := (Width) div FDateSlots;
      N := FStartDay; X := 2;
       while X < Width-2 do
        begin
           {draws the slash}
          Pen.Mode := pmCopy;
          Pen.Style := psSolid;
          Pen.Color := FLineColor;
          MoveTo(X,(Height div 3));
          if (N = 1) then
          begin
            MoveTo(X, Height-(Height div 3));
          end
          else
              LineTo(X, Height-(Height div 3));
          if N = 0 then
             N:=1;
          if N > FDateSlots then
             N := 1;
          N2 := N;
          if N = 1 then N2 := 1;

          if FDateText <> '' then
          begin
            DateVal :=  StrtoDate(FDateText);
            DecodeDate(DateVal,Year, Month, Day);
          end;

          if ((N <= (DaysInMonth(Month, Year))) or (DateOverride)) then
          begin
            if ((FStartDay - FDateSlots) < 0) or (N < FDateSlots) then
            begin
              {Writes number}
              if FDisplaySelectCaption = FALSE then
              begin
                DateVal :=  StrtoDate(FDateText);
                DecodeDate(DateVal,Year, Month, Day);
                DateVal := EncodeDate(Year, Month, N);
                DecodeDate(DateVal,Year, Month, Day);
                CurrDay := DayOfWeek(DateVal);

                if ((CurrDay = 1) or (CurrDay = 7)) then
                begin
                  {Create FWeekendColor}
                  Font.Color := FWeekendColor;
                end
                else
                   begin
                     Font.Color := FWeekdayColor;
                   end;
                TextOut(X + 1,
                      PenPos.Y -(Height div 6),
                      m_DayTitles[CurrDay]);
              end;
              Font.Color := FDateColor;
              if N < 9 then
                 TextOut(X+4,
                      PenPos.Y -((Height div 8)+4),
                      IntToStr(N2))
              else
                  TextOut(X+1,
                      PenPos.Y -((Height div 8)+4),
                      IntToStr(N2));
            end;
          end;
          Inc(N);
          if N = FDateSlots then
          begin
             N := FDateSlots;
          end;
          X := X + Sections;

         end;{end of while}
      {draw the last line}
      Pen.Mode := pmCopy;
      Pen.Style := psSolid;
      Pen.Color := FLineColor;
      MoveTo(X,(Height div 3));
      LineTo(X, Height-(Height div 3));

      with ClientRect do
       begin
         {Create Title Bar}
         Pen.Style := psSolid;
         Brush.Color := FDateBoxColor;
         NewRect := ClientRect;
         NewRect.Top := 2;
{       NewRect.Bottom := Height div 3;}
         NewRect.Bottom := (Height div 3);
         NewRect.Left := 3;
         NewRect.Right := Width;
         FillRect(NewRect);
         SavedColor := Font.Color;
         Font.Assign(FDateTextFont);
         if FDisplaySelectCaption then FMonthText := 'Select Date(s)'
         else RestoreMonthText;
         TextOut((Width div 2)-(TextWidth(FMonthText) div 2), 4, FMonthText);

         Font.Color := SavedColor;
         Brush.Color := Color;
         Pen.Color := FShadowColor;

         DrawButtons;

         MoveTo(1,Height div 3);
         LineTo(Width-2,Height div 3);
         Pen.Color := FLineColor;
         MoveTo(1,(Height div 3)-1);
         LineTo(Width-2,(Height div 3)-1);
         MoveTo(1,Height - (Height div 3));
         LineTo(Width-2,Height - (Height div 3));
         Pen.Color := FHighlightColor;
         MoveTo(Width-1,1);
         LineTo(Width-1,Height-1);
         MoveTo(Width-1,Height-1);
         LineTo(1,Height-1);
         Pen.Color := FShadowColor;
         MoveTo(0,0);
         LineTo(Width-1,0);
         MoveTo(1,1);
         LineTo(1,Height-1);
       end;
     DrawSelectedDateSlots;
   end;
end;

{******************** GetPreviousMonth ********************
 **********************************************************}
procedure TDateBar.PrevMonth;
var
 CurrYear, CurrMonth, CurrDay : Word;
 StartYear, StartMonth, StartDay : Word;
 EndYear, EndMonth, EndDay : Word;
 DateVal, CurrDateVal, StartDateVal, EndDateVal : TDateTime;
begin
  if FDateText <> '' then
  begin
    CurrDateVal :=  StrtoDate(FDateText);
    DecodeDate(CurrDateVal,CurrYear, CurrMonth, CurrDay);
  end
  else
     begin
       if DateOverride then
          exit;
       CurrDateVal :=  Date;
       DecodeDate(CurrDateVal,CurrYear, CurrMonth, CurrDay);
     end;

  if FEndDateText <> '' then
  begin
    EndDateVal :=  StrtoDate(FEndDateText);
    DecodeDate(EndDateVal,EndYear, EndMonth, EndDay);
  end
  else
     begin
//       EndDateVal :=  Date;
     end;

  if FStartDateText <> '' then
  begin
    StartDateVal :=  StrtoDate(FStartDateText);
    DecodeDate(StartDateVal,StartYear, StartMonth, StartDay);
  end
  else
     begin
       StartDateVal :=  Date;
       DecodeDate(StartDateVal,StartYear, StartMonth, StartDay);
     end;

  if ((StartDateVal < CurrDateVal) and (CurrMonth <> StartDateVal)) then
  begin
    dec(CurrMonth);
    if CurrMonth = 0 then
    begin
      CurrMonth := 12;
      dec(CurrYear);
    end;

    DateVal := EncodeDate(CurrYear, CurrMonth, 1);
    SetDateText(DatetoStr(DateVal));
    if Assigned(FOnPrevMonth) then OnPrevMonth(Self);
  end
  else
       SetDateText(DatetoStr(CurrDateVal))
end;

{******************** Get Next Month **********************
 **********************************************************}
procedure TDateBar.NextMonth;
var
 CurrYear, CurrMonth, CurrDay : Word;
 StartYear, StartMonth, StartDay : Word;
 EndYear, EndMonth, EndDay : Word;
 DateVal, CurrDateVal, StartDateVal, EndDateVal : TDateTime;
begin
  if FDateText <> '' then
  begin
    CurrDateVal :=  StrtoDate(FDateText);
    DecodeDate(CurrDateVal,CurrYear, CurrMonth, CurrDay);
  end
  else
     begin
       CurrDateVal :=  Date;
       DecodeDate(CurrDateVal,CurrYear, CurrMonth, CurrDay);
     end;

  if FEndDateText <> '' then
  begin
    EndDateVal :=  StrtoDate(FEndDateText);
    DecodeDate(EndDateVal,EndYear, EndMonth, EndDay);
  end
  else
     begin
       EndDateVal :=  Date;
     end;

  if FStartDateText <> '' then
  begin
    StartDateVal :=  StrtoDate(FStartDateText);
    DecodeDate(StartDateVal,StartYear, StartMonth, StartDay);
  end
  else
     begin
       StartDateVal :=  Date;
       DecodeDate(StartDateVal,StartYear, StartMonth, StartDay);
     end;

  if ((CurrDateVal < EndDateVal) and (CurrMonth <> EndMonth)) then
  begin
    inc(CurrMonth);
    if CurrMonth > 12 then
    begin
      CurrMonth := 1;
      inc(CurrYear);
    end;

    DateVal := EncodeDate(CurrYear, CurrMonth, 1);
    SetDateText(DatetoStr(DateVal));
    if Assigned(FOnNextMonth) then OnNextMonth(Self);
  end
  else
      SetDateText(DatetoStr(CurrDateVal))
end;

(*********************************************
Determine which token the user is on and highlight
the entire text of that token.
*********************************************)
procedure TDateBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  {Check if mouse was pressed in Left button area}
  if PointInRect(GetLeftButtonRect, X, Y) then
     begin
       PrevMonth;
       Refresh;
     end;

  {Check if mouse was pressed in Right button area}
  if PointInRect(GetRightButtonRect, X, Y) then
     begin
       NextMonth;
       Refresh;
     end;

  AnchorX := X; CurX := X;
  AnchorY := Height - (Height div 3)+1; CurY := Height-2;
  if (Y > (Height div 3)) and ((X > 1) or (X < Width-1)) then Bounding := true;
  inherited MouseDown( Button, Shift, X, Y );
end;

procedure TDateBar.RefreshSelect;
begin
 with Canvas do
  begin
   Brush.Color := FSelectedColor;
   FillRect(RefreshRect);
   DrawSegments;
  end;
end;

procedure TDateBar.DrawSelect(DrawRect: TRect);
begin
 RefreshRect := DrawRect;
 with Canvas do
  begin
   Brush.Color := FSelectedColor;
   FillRect(RefreshRect);
  end;
end;

(*********************************************
Determine which token the user is on and highlight
the entire text of that token.
*********************************************)
procedure TDateBar.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: integer );
var
 SelectedRect: TRect;
 Sections,
 N: Integer;
begin
  if not FMultiDaySelect then
     begin
        X := AnchorX;
        Y := AnchorY;
     end;
  if Bounding then
   with Canvas do
    begin
     Bounding := false;
     Pen.Style := psDot;
     Pen.Mode := pmNot;
     Brush.Style := FSelectBrush;
     Rectangle(AnchorX, AnchorY, CurX, CurY);
     Pen.Mode := pmCopy;
     Brush.Style := bsSolid;
     if ([ssCtrl] <= Shift) then
      begin
      end
     else
      begin
       Brush.Color := Color;
       SelectedRect.Top := AnchorY;
       SelectedRect.Left := 2;
       SelectedRect.Bottom := CurY;
       SelectedRect.Right := Width;
       FillRect(SelectedRect);
      end;
      Sections := (Width) div FDateSlots;
      N := FStartDay;
      X := 2;
       while X < Width-1 do
        begin
         Pen.Color := FLineSelectColor;
         Pen.Style := psSolid;
         MoveTo(X,Height - (Height div 3));
         LineTo(X, Height-2);

         Brush.Color := FSelectedColor;
         SelectedRect.Top := AnchorY+1;
         SelectedRect.Bottom := CurY-1;

          begin
           if (AnchorX >= X) and (AnchorX < X+Sections) then
               AnchorX := X;

           if (CurX >= X) and (CurX < X+Sections) then
               CurX := X;
          end;

         if CurX > AnchorX then
          begin
           SelectedRect.Left := AnchorX;
           SelectedRect.Right := CurX;
          end
         else
          begin
           SelectedRect.Left := CurX;
           SelectedRect.Right := AnchorX;
          end;

         Inc(N);
         X := X + Sections;
        end;{end of While}
     if CurX > AnchorX then
      begin
        {Assign left and right}
        CurX := CurX + (Sections);
        SelectedRect.Left := AnchorX;
        SelectedRect.Right := CurX;
      end
     else
      begin
       AnchorX := AnchorX + (Sections);
{       CurX := CurX + (Sections);}
       SelectedRect.Left := CurX;
       SelectedRect.Right := AnchorX;
      end;
     DrawSelect(SelectedRect);
     if ([ssCtrl] <= Shift) then
      begin
       FDateListStart.Add(GetDaySlot(SelectedRect.Left div (Sections)+1));
       FDateListEnd.Add(GetDaySlot(SelectedRect.Right div (Sections)));
      end
     else
      begin
       FDateListStart.Clear;
       FDateListEnd.Clear;
       FDateListStart.Add(GetDaySlot(SelectedRect.Left div (Sections)+1));
       FDateListEnd.Add(GetDaySlot(SelectedRect.Right div (Sections)));
      end;
     DrawSegments;
   end;
  if Assigned(FOnDateSelected) then OnDateSelected(Self);
  inherited MouseUp( Button, Shift, X, Y );
end;

(*********************************************
Determine which token the user is on and highlight
the entire text of that token.
*********************************************)
procedure TDateBar.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if not FMultiDaySelect then
     begin
        X := AnchorX;
        Y := AnchorY;
     end;
  if (Bounding) then
  with Canvas do
   begin
    Pen.Mode := pmNot;
    Pen.Width := 1;
    Pen.Style := psDot;
    Brush.Style := FSelectBrush;
    Rectangle(AnchorX, AnchorY, CurX, CurY);
    CurX := X;
    if CurX < 2 then CurX := 2;
    if CurX > Width-2 then CurX := Width-2;
    Rectangle(AnchorX, AnchorY, CurX, CurY);
   end;
  inherited MouseMove( Shift, X, Y );
end;

procedure TDateBar.DrawSegments;
var
 Sections, N, X : Word;
begin
 with Canvas do
  begin
   Sections := (Width) div FDateSlots;
{   Sections := (Width) div FDateSlots;}
   N := FStartDay; X := 2;
    while X < Width-1 do
     begin
       Pen.Mode := pmCopy;
       Pen.Style := psSolid;
       Pen.Color := Color;
       Pen.Width := 3;
       MoveTo(X,Height - (Height div 3)+1);
       LineTo(X, Height-4);

       Pen.Width := 1;
       Pen.Color := FLineSelectColor;
       MoveTo(X,Height - (Height div 3));
       LineTo(X, Height-2);

       Inc(N);
       X := X + Sections;
     end;
     {Last line end of the rect}
     X := Width - 2;
     Pen.Width := 1;
     Pen.Color := FLineSelectColor;
     MoveTo(X,Height - (Height div 3));
     LineTo(X, Height-2);

    Pen.Color := FShadowColor;
    MoveTo(1,Height - (Height div 3));
    LineTo(1, Height);
    Pen.Color := FLineColor;
    MoveTo(1,Height-(Height div 3));
    LineTo(Width-2,Height-(Height div 3));
   end;
end;

procedure TDateBar.SeTDateBarColor(Value: TColor);
begin
  if FBackgroundColor <> Value then
   begin
     FBackgroundColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetLineColor(Value: TColor);
begin
  if FLineColor <> Value then
   begin
     FLineColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetShadowColor(Value: TColor);
begin
  if FShadowColor <> Value then
   begin
     FShadowColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
   begin
     FHighlightColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetDateBoxColor(Value: TColor);
begin
  if FDateBoxColor <> Value then
   begin
     FDateBoxColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetLineSelectColor(Value: TColor);
begin
  if FLineSelectColor <> Value then
   begin
     FLineSelectColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetSelectedColor(Value: TColor);
begin
  if FSelectedColor <> Value then
   begin
     FSelectedColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetDateColor(Value: TColor);
begin
  if FDateColor <> Value then
   begin
     FDateColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetWeekendColor(Value: TColor);
begin
  if FWeekendColor <> Value then
   begin
     FWeekendColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetWeekdayColor(Value: TColor);
begin
  if FWeekdayColor <> Value then
   begin
     FWeekdayColor := Value;
   end;
 Invalidate;
end;

procedure TDateBar.SetDateSlots(Value: Integer);
begin
 if FDateSlots <> Value then
  begin
   if Value < 1 then
    begin
     FDateSlots := 1;
    end
   else
   if (Value > 31) then
    begin
     FDateSlots := 31;
    end
   else
    begin
     FDateSlots := Value;
    end;
  end;
 Invalidate;
end;

procedure TDateBar.SetStartDay(Value: Integer);
begin
 if FStartDay <> Value then
  begin
   if (Value > 31) or (Value < 1) then
   begin
       FStartDay := 1;
   end
   else
      begin
         FStartDay := Value;
      end;
  end;
 Invalidate;
end;

function TDateBar.GetDaySlot(Value: Integer): String;
var
 NewTime: TDateTime;
 Year, Month, Day, LastDayInMonth : Word;
begin
  if Value = 0 then
     Value := 1;
  if DateOverride then
  begin
     LastDayInMonth := FDateSlots;
     NewTime := Date;
     decodeDate(NewTime, Year, Month, Day);
  end
  else
     begin
       {Decode CurrentDate and add month & year}
       DecodeDate(StrToDate(FDateText), Year, Month, Day);
       LastDayInMonth := DaysInMonth(Month, Year);
     end;
  if (Value > LastDayInMonth) then
   begin
     Value := LastDayInMonth;
     Invalidate;
   end;
  Result := IntToStr(Month)+'/'+InttoStr(Value)+'/'+IntToStr(Year);
end;

function TDateBar.GetSlotNumber(Value: String): Integer;
var
 DateVal : TDateTime;
 Year, Month, Day : Word;
 Year2, Month2, Day2: Word;
 SelYear, SelMonth, SelDay : Word;
 CurrYear, CurrMonth, CurrDay : Word;
 StartYear, StartMonth, StartDay : Word;
 EndYear, EndMonth, EndDay : Word;
 SelVal, CurrVal, StartVal, EndVal : TDateTime;
begin
  Result := -1;
  if FDateText <> '' then
  begin
    try
       DateVal :=  StrtoDate(Value);
    except
         DateVal := 0;
    end;

    SelVal :=  StrtoDate(Value);
    DecodeDate(SelVal,SelYear, SelMonth, SelDay);
    CurrVal :=  StrtoDate(FDateText);
    DecodeDate(CurrVal,CurrYear, CurrMonth, CurrDay);
    StartVal :=  StrtoDate(FStartDateText);
    DecodeDate(StartVal,StartYear, StartMonth, StartDay);
    EndVal :=  StrtoDate(FEndDateText);
    DecodeDate(EndVal,EndYear, EndMonth, EndDay);

    if ((SelVal < StartVal) or (SelVal > EndVal)) then
    begin
       Result := -1;
       Exit;
    end;

    if DateOverride then
       begin
          DateVal := Date;
          DecodeDate(DateVal,Year, Month, Day);
          DecodeDate(DateVal,Year2, Month2, Day2);
       end
    else
       begin
         DecodeDate(DateVal,Year, Month, Day);
         DateVal :=  StrtoDate(FDateText);
         DecodeDate(DateVal,Year2, Month2, Day2);
       end;
  end;
  if Value <> '' then
  begin
    if (Day > (DaysInMonth(Month, Year))) then
    begin
      Result := DaysInMonth(Month, Year)
    end
    else
         Result := Day;
  end;
  if ((Month <> Month2) or (Year <> Year2)) then Result := -1;
end;

procedure TDateBar.SetDateText(Value: String);
var
 Year, Month, Day: Word;
 DateVal : TDateTime;
begin
  if (Value <> '') then
  begin
    if (FDateText <> Value) then
    begin
      DateOverride := False;
      FDateText := Value;
      DateVal :=  StrtoDate(FDateText);
      DecodeDate(DateVal,Year, Month, Day);
      FMonthText := m_MonthTitles[Month] + ' ' + InttoStr(Year);
    end;
  end
  else
     begin
     //  DateOverride := True;
       FDateText := Value;
       FMonthText := 'Select Date(s)';
     end;
  Invalidate;
end;

procedure TDateBar.RestoreMonthText;
var
 Year, Month, Day: Word;
 DateVal : TDateTime;
begin
      DateVal :=  StrtoDate(FDateText);
      DecodeDate(DateVal,Year, Month, Day);
      FMonthText := m_MonthTitles[Month] + ' ' + InttoStr(Year);
end;

procedure TDateBar.SetStartDateText(Value: String);
begin
  if (Value <> FStartDateText) then
  begin
    if (FStartDateText <> Value) then
     begin
      FStartDateText := Value;
     end;
  end;
end;

procedure TDateBar.SetEndDateText(Value: String);
begin
  if (Value <> FEndDateText) then
  begin
    if (FEndDateText <> Value) then
     begin
      FEndDateText := Value;
     end;
  end;
end;

procedure TDateBar.SetCenterDay(Value: Boolean);
begin
 if FCenterDay <> Value then
  begin
   FCenterDay := Value;
  end;
 Invalidate;
end;

procedure TDateBar.SetDisplaySelectCaption(Value: Boolean);
begin
 if FDisplaySelectCaption <> Value then
  begin
   FDisplaySelectCaption := Value;
  end;
 Invalidate;
end;

procedure TDateBar.SetDateTextFont(Value: TFont);
begin
  if FDateTextFont <> Value then
   begin
     FDateTextFont.Assign( Value );
   end;
 Invalidate;
end;

procedure TDateBar.SetAbout(Value: TAbout);
begin
  if FAbout <> Value then
     FAbout.Assign( Value );
     {Create default greeting list}
     MessageBeep(0);
     FAbout.Clear;
     FAbout.Add('TDateBar - Version 1.0');
     FAbout.Add('-----------------------------');
     FAbout.Add(' ');
     FAbout.Add('No other compares to Delphi!');
     FAbout.Add(' ');
     FAbout.Add('Copyright  1996 by Jim Wilhelm');
end;

procedure TDateBar.SetDateListStart(Value: TDateList);
begin
  if FDateListStart <> Value then
   begin
    FDateListStart.Assign( Value );
    Invalidate;
   end;
end;

procedure TDateBar.SetDateListEnd(Value: TDateList);
begin
  if FDateListEnd <> Value then
   begin
    FDateListEnd.Assign( Value );
    Invalidate;
   end;
end;

procedure TDateBar.SetDayList(Value: TDayList);
begin
  if FDayList <> Value then
   begin
    FDayList.Assign( Value );
   end;
 Invalidate;
end;

procedure TDateBar.SetSelectByDay(Value: Boolean);
begin
  if FSelectByDay <> Value then
   begin
    FSelectByDay :=  Value;
   end;
 Invalidate;
end;

procedure TDateBar.SelectByDay;
var
 Year, Month, Day, CurrDay : Word;
 DateVal : TDateTime;
 Days, i, j : Integer;
begin
  FDateListStart.Clear;
  FDateListEnd.Clear;
  if DateOverride then
  begin
    Days := FDateSlots;
//    DateVal := Date;
  end
  else
     begin
       DateVal :=  StrtoDate(FDateText);
       DecodeDate(DateVal,Year, Month, Day);
       Days :=  DaysInMonth(Month, Year);
     end;
  for i:=1 to Days do
  begin
     DateVal := EncodeDate(Year, Month, i);
     DecodeDate(DateVal,Year, Month, Day);
     CurrDay := DayOfWeek(DateVal);
     for j:=0 to FDayList.Count-1 do
     begin
       if StrtoInt(FDayList[j]) = CurrDay then
       begin
         FDateListStart.Add(IntToStr(Month)+'/'+InttoStr(i)+'/'+IntToStr(Year));
         FDateListEnd.Add(IntToStr(Month)+'/'+InttoStr(i)+'/'+IntToStr(Year));
       end;
     end;
  end;
 Invalidate;
end;

procedure TDateBar.DrawSelectedDateSlots;
var
 SelectedRect: TRect;
 SlotStart,
 SlotEnd,
 SlotID: Integer;
begin
 if FDateListStart.Count <> FDateListEnd.Count then Exit;
 AnchorY := Height - (Height div 3)+1; CurY := Height-2;
 SelectedRect.Top := AnchorY+1;
 SelectedRect.Bottom := CurY-1;

 for SlotID := 0 to FDateListStart.Count - 1 do
  begin
    SlotStart := GetSlotNumber(FDateListStart[SlotID]);
    SlotEnd := GetSlotNumber(FDateListEnd[SlotID]);
   if ((SlotStart <> -1) and  (SlotEnd <> -1)) then
   begin
     if SlotStart > SlotEnd then
     begin
       SlotEnd := ((Width) div (FDateSlots));
     end;
     SelectedRect.Left := ((SlotStart*((Width-4) div FDateSlots))- ((Width-4) div FDateSlots))+2;
     SelectedRect.Right := (SlotEnd*((Width-4) div FDateSlots))+2;
     DrawSelect(SelectedRect);
   end;
  end;
 DrawSegments;
end;

destructor TDateBar.Destroy;
begin
  FAbout.Free;
  FDateListStart.Free;
  FDateListEnd.Free;
  FDayList.Free;
  FDateTextFont.Free;
  inherited Destroy;
end;
end.

