unit Timebar;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Menus, 
  Classes, Graphics, Controls, Forms, Dialogs, 
  StdCtrls, ExtCtrls;

const
 MAXNUMBEROFROWS = 100;
 TEXT_INDENT = 2;

type

  TTimeList = class(TStringList)
   end;
  TCopyright = class(TStringList)
   end;
  TTimeSlots = class(TTimeList)
   end;
  TAbout = class(TCopyright)
   end;
  TTimeBar = class(TPaintBox)
  private
    FAbout: TAbout;
    FButtonIndent: Integer;
    FButtonHeight: Integer;
    FTimeListStart: array[1..MAXNUMBEROFROWS] of TTimeSlots;
    FTimeListEnd: array[1..MAXNUMBEROFROWS] of TTimeSlots;
    FSelectBrush: TBrushStyle;
    FNumberOfRows: Integer;
    FDateText: String;
    FDateText2: String;
    FOnTimeSelected: TNotifyEvent;
    FBackgroundColor: TColor;
    FSelectedColor: TColor;
    FLineColor: TColor;
    FShadowColor: TColor;
    FHighlightColor: TColor;
    FDateBoxColor: TColor;
    FLineSelectColor: TColor;
    FDateTextFont: TFont;
    FTimeSlots: Integer;
    FTimeSlotIndex: Integer;
    FStartHour: Integer;
    FFormatTime: Boolean;
    FAMPM: Boolean;
    FHalfHour: Boolean;
    FCenterTime: Boolean;
    FOnNextHour : TNotifyEvent;
    FOnPrevHour : TNotifyEvent;
    FShowButtons: Boolean;
    Bounding: Boolean;
    RowHeight: Integer;
    AnchorX: Integer;
    AnchorY: Integer;
    CurX: Integer;
    CurY: Integer;
    RefreshRect: TRect;
    procedure SetAbout(Value: TAbout);
    procedure SeTTimeBarColor(Value: TColor);
    procedure SetLineColor(Value: TColor);
    procedure SetShadowColor(Value: TColor);
    procedure SetHighlightColor(Value: TColor);
    procedure SetDateBoxColor(Value: TColor);
    procedure SetLineSelectColor(Value: TColor);
    procedure SetSelectedColor(Value: TColor);
    procedure SetNumberOfRows(Value: Integer);
    procedure SetStartHour(Value: Integer);
    procedure SetTimeSlots(Value: Integer);
    procedure SetTimeSlotIndex(Value: Integer);
    procedure SetDateTextFont(Value: TFont);
    procedure SetAMPM(Value: Boolean);
    procedure SetHalfHour(Value: Boolean);
    procedure SetCenterTime(Value: Boolean);
    procedure SetDateText(Value: String);
    procedure SetTimeSlotsStart(Value: TTimeSlots);
    procedure SetTimeSlotsEnd(Value: TTimeSlots);
    function  GetTimeSlotsStart: TTimeSlots;
    function  GetTimeSlotsEnd: TTimeSlots;
    procedure DrawSegments;
    procedure RefreshSelect;
    procedure DrawSelect(DrawRect: TRect);
    procedure DrawSelectedTimeSlots;
    function  FormatAsTime(Value: String): String;
  protected
    procedure DrawButtons;
    function  GetLeftButtonRect : TRect;
    function  GetRightButtonRect : TRect;
    procedure NextHour;
    procedure PrevHour;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: integer ); override;
    procedure WMPaintItem(var Message: TWMPaint); message WM_PAINT;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    function  GetTimeSlotA(Value: Integer): String;
    function  GetTimeSlotB(Value: Integer): String;
    function  GetSlotNumber(Value: String): Integer;
  published
    property About: TAbout read FAbout write SetAbout;
    property NumberOfRows: Integer read FNumberOfRows write SetNumberOfRows;
    property Color : TColor read FBackgroundColor write SeTTimeBarColor;
    property LineColor : TColor read FLineColor write SetLineColor;
    property ShadowColor : TColor read FShadowColor write SetShadowColor;
    property HighlightColor : TColor read FHighlightColor write SetHighlightColor;
    property DateBoxColor: TColor read FDateBoxColor write SetDateBoxColor;
    property LineSelectColor: TColor read FLineSelectColor write SetLineSelectColor;
    property DateTextFont: TFont read FDateTextFont write SetDateTextFont;
    property Date: String read FDateText write SetDateText;
    property TimeSlots: Integer read FTimeSlots write SetTimeSlots;
    property StartHour: Integer read FStartHour write SetStartHour;
    property AMPMFormat: Boolean read FAMPM write SetAMPM;
    property FormatTime: Boolean read FFormatTime write FFormatTime;
    property HalfHourIncrements: Boolean read FHalfHour write SetHalfHour;
    property CenterTime: Boolean read FCenterTime write SetCenterTime;
    property TimeSlotIndex: Integer read FTimeSlotIndex write SetTimeSlotIndex;
    property SelectedStart: TTimeSlots read GetTimeSlotsStart write SetTimeSlotsStart;
    property SelectedEnd: TTimeSlots read GetTimeSlotsEnd write SetTimeSlotsEnd;
    property SelectedColor: TColor read FSelectedColor write SetSelectedColor;
    property SelectBrushStyle: TBrushStyle read FSelectBrush write FSelectBrush;
    property OnTimeSelected : TNotifyEvent read FOnTimeSelected write FOnTimeSelected;
    property OnNextHour : TNotifyEvent read FOnNextHour write FOnNextHour;
    property OnPrevHour : TNotifyEvent read FOnPrevHour write FOnPrevHour;
    property ShowButtons: Boolean read FShowButtons write FShowButtons;
    property ButtonHeight: Integer read FButtonHeight write FButtonHeight;
    property ButtonIndent: Integer read FButtonIndent write FButtonIndent;
    property Align;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property Left;
    property Name;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Tag;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional',[TTimeBar]);
end;

constructor TTimeBar.Create(AOwner:TComponent);
var
 I: Integer;
begin
  inherited Create(AOwner);
  Height := 58;
  Width := 400;
  FNumberOfRows := 1;
  FButtonHeight := 14;
  FButtonIndent := 2;
  ShowHint := TRUE;
  Hint := 'Hold the [Ctrl] key down to select multiple times';
  FBackgroundColor := clBtnFace;
  FShadowColor := clBtnShadow;
  FSelectedColor := clHighlight;
  FHighlightColor := clBtnHighlight;
  FLineColor := clBtnText;
  FDateBoxColor := clActiveCaption;
  FLineSelectColor := clGray;
  FTimeSlots := 12;
  FTimeSlotIndex := 1;
  Font.Name := 'MS Sans Serif';
  Font.Size := 8;
  FFormatTime := TRUE;
  FSelectBrush := bsSolid;
  FDateTextFont := TFont.Create;
  FDateTextFont.Assign(Font);
  FDateTextFont.Style := [fsBold];
  FDateTextFont.Color := clCaptionText;
  for I := 1 to MAXNUMBEROFROWS do
   begin
    FTimeListStart[I] := TTimeSlots.Create;
    FTimeListEnd[I] := TTimeSlots.Create;
   end;
  FDateText := DateToStr(Now);
  FAbout := TAbout.Create;
  FAbout.Clear;
  FAbout.Add('TTimeBar - Version 1.1');
  FAbout.Add('-----------------------------');
  FAbout.Add(' ');
  FAbout.Add('Copyright  1996 by Jim Wood');
end;

{********************* Get Left Button Rectangle ******************
 ***** Get the rectangle used for the left button.            *****
 ******************************************************************}
function TTimeBar.GetLeftButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Left Button Rectangle}
   TempRect.Top := ClientRect.Top + FButtonIndent;
   TempRect.Bottom := TempRect.Top + FButtonHeight;
   TempRect.Left := FButtonIndent + TEXT_INDENT;
   TempRect.Right := TempRect.Left + FButtonHeight;

   Result := TempRect;
end;

{******************** Get Right Button Rectangle ******************
 ***** Get the rectangle used for the right button.           *****
 ******************************************************************}
function TTimeBar.GetRightButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Right Button Rectangle}
   TempRect.Top := ClientRect.Top + FButtonIndent;
   TempRect.Bottom := TempRect.Top + FButtonHeight;
   TempRect.Right := Width-(FButtonIndent + TEXT_INDENT);
   TempRect.Left := TempRect.Right - FButtonHeight;

   Result := TempRect;
end;

{******************** GetPreviousMonth ********************
 **********************************************************}
procedure TTimeBar.PrevHour;
begin
 if FStartHour = 0 then
    begin
     MessageBeep(MB_OK);
     Exit;
    end;
 SetStartHour(FStartHour-1);
 if Assigned(FOnPrevHour) then OnPrevHour(Self);
end;

{******************** Get Next Month **********************
 **********************************************************}
procedure TTimeBar.NextHour;
begin
 SetStartHour(FStartHour+1);
 if 23-FStartHour < FTimeSlots-1 then
    begin
       SetStartHour(FStartHour-1);
       MessageBeep(MB_OK);
       Exit;
    end;
 if Assigned(FOnNextHour) then OnNextHour(Self);
end;

procedure TTimeBar.DrawButtons;
var
  LeftButtonRect: TRect;
  RightButtonRect : TRect;
  OldStyle : TBrushStyle;
begin
  if FShowButtons then
  with Canvas do
     begin
        LeftButtonRect := GetLeftButtonRect;
        RightButtonRect := GetRightButtonRect;

        { Select Black Pen}
        Pen.Style := psSolid;
        Pen.Width := 1;
        Pen.Color := clBlack;
        Brush.Color := Color;

        { Draw Button Outlines }
        Rectangle( LeftButtonRect.Left, LeftButtonRect.Top, LeftButtonRect.Right, LeftButtonRect.Bottom );
        Rectangle( RightButtonRect.Left, RightButtonRect.Top, RightButtonRect.Right, RightButtonRect.Bottom );

        { Create Embossed effect - Outline left & upper in white}
        Pen.Color := clWhite;
        MoveTo( LeftButtonRect.Left + 1, LeftButtonRect.Bottom - 2 );
        LineTo( LeftButtonRect.Left + 1, LeftButtonRect.Top + 1 );
        LineTo( LeftButtonRect.Right - 2, LeftButtonRect.Top + 1 );

        MoveTo( RightButtonRect.Left + 1, RightButtonRect.Bottom - 2 );
        LineTo( RightButtonRect.Left + 1, RightButtonRect.Top + 1 );
        LineTo( RightButtonRect.Right - 2, RightButtonRect.Top + 1 );

        { Create Embossed effect - Outline right & bottom in gray }
        Pen.Color := clGray;
        MoveTo( LeftButtonRect.Right -2, LeftButtonRect.Top +  1 );
        LineTo( LeftButtonRect.Right - 2, LeftButtonRect.Bottom - 2 );
        LineTo( LeftButtonRect.Left + 1, LeftButtonRect.Bottom - 2 );

        MoveTo( RightButtonRect.Right - 2, RightButtonRect.Top + 1 );
        LineTo( RightButtonRect.Right - 2, RightButtonRect.Bottom - 2 );
        LineTo( RightButtonRect.Left + 1, RightButtonRect.Bottom - 2 );

        {Draw Arrow}
        Brush.Color := clBlack;
        OldStyle :=Brush.Style;
        Brush.Style := bsSolid;
        Polygon([Point(LeftButtonRect.Right - 5,LeftButtonRect.Top + 3),
                 Point(LeftButtonRect.Right - 5,LeftButtonRect.Bottom - 4),
                 Point(LeftButtonRect.Left + 3,LeftButtonRect.Top + 7)]);

        Polygon([Point(RightButtonRect.Left + 4,RightButtonRect.Top + 3),
                 Point(RightButtonRect.Left + 4,RightButtonRect.Bottom - 4),
                 Point(RightButtonRect.Right - 4,RightButtonRect.Top + 7)]);

        Brush.Color :=clSilver;
        Brush.Style := OldStyle;
        Pen.Color := clBlack;
     end;
end;

procedure TTimeBar.WMPaintItem(var Message: TWMPaint);
var
 Sections, N, N2, X: Word;
// Day : Word;
 NewRect: TRect;
// DrawRow: Integer;
 SavedColor: TColor;
 TrailText: String;
begin
 while (((FTimeSlots * 4 )*((Width-4) div (FTimeSlots * 4))) <> (Width-4)) and (Width > 0) do
  begin
   Width := Width - 1;
  end;
 while (((FNumberOfRows+2)*((Height-4) div (FNumberOfRows+2))) <> (Height-4)) and (Height > 0) do
  begin
   Height := Height - 1;
  end;

 Canvas.Font.Assign(Font);
 with Canvas do
  begin
   if Bounding then
      begin
         Pen.Style := psSolid;
         Pen.Mode := pmCopy;
         Brush.Style := bsSolid;
      end;
   Pen.Color := clBlack;
   Brush.Color := FBackgroundColor;
    FillRect(ClientRect);
     with ClientRect do
      begin
       Rectangle(Left+2, Top+1, Right-1, Bottom-1);
      end;
      Sections := (Width-4) div FTimeSlots;
      N := FStartHour; X := 2;
        while X < Width-2 do
         begin

           Pen.Mode := pmCopy;
           Pen.Style := psSolid;
           Pen.Color := FLineColor;
           RowHeight := (Height-4) div (FNumberOfRows+2);
           MoveTo(X,RowHeight);
           LineTo(X,RowHeight*2);
           if not FHalfHour then
            begin
             MoveTo(X+(Sections div 4),RowHeight+(RowHeight div 2)+(RowHeight div 4));
             LineTo(X+(Sections div 4),RowHeight*2);
            end;
           MoveTo(X+(Sections div 2),RowHeight+(RowHeight div 2));
           LineTo(X+(Sections div 2),RowHeight*2);
           if not FHalfHour then
            begin
             MoveTo(X+((Sections div 2)+(Sections div 4)),RowHeight+(RowHeight div 2)+(RowHeight div 4));
             LineTo(X+((Sections div 2)+(Sections div 4)),RowHeight*2);
            end;

           if FAMPM then
            begin
             N2 := N;
             if (N > 11) and (N < 24) then
              begin
               if N > 12 then N2 := N-12;
               TrailText := ' PM'
              end
             else
             if (N < 12) then
              begin
               TrailText := ' AM'
              end
             else
             if N > (23+FTimeSlots) then
              begin
               N := 0;
               N2 := N;
               TrailText := ' AM'
              end;
             if N = 0 then N2 := 12;
             if (FHalfHour = True) and (FCenterTime = True) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut((X+(Sections div 2))-(TextWidth(IntToStr(N2)+':30'+TrailText)div 2),
                       PenPos.Y-RowHeight+2,
                       IntToStr(N2)+':30'+TrailText);
              end
             else
             if (FHalfHour = False) and (FCenterTime = True) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut((X+(Sections div 2))-(TextWidth(IntToStr(N2)+':30'+TrailText)div 2),
                       PenPos.Y-RowHeight+2,
                       IntToStr(N2)+':30'+TrailText);
              end
             else
             if (FHalfHour = True) and (FCenterTime = False) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut(X + 1,
                       PenPos.Y-RowHeight+2,
                       IntToStr(N2)+':00'+TrailText);
              end
             else
             if (FHalfHour = False) and (FCenterTime = False) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut(X + 1,
                       PenPos.Y-RowHeight+2,
                       IntToStr(N2)+':00'+TrailText);
              end;
            end
           else
            begin
             if N > (23+FTimeSlots) then N := 0;
             if (FHalfHour = True) and (FCenterTime = True) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut((X+(Sections div 2))-(TextWidth(IntToStr(N)+':30')div 2),
                       PenPos.Y-RowHeight+2,
                       IntToStr(N)+':30');
              end
             else
             if (FHalfHour = False) and (FCenterTime = True) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut((X+(Sections div 2))-(TextWidth(IntToStr(N)+':30')div 2),
                       PenPos.Y-RowHeight+2,
                       IntToStr(N)+':30');
              end
             else
             if (FHalfHour = True) and (FCenterTime = False) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut(X + 1,
                       PenPos.Y-RowHeight+2,
                       IntToStr(N)+':00');
              end
             else
             if (FHalfHour = False) and (FCenterTime = False) then
              begin
               if ((FStartHour - FTimeSlots) < 0) or (N < 24) then
               TextOut(X + 1,
                       PenPos.Y-RowHeight+2,
                       IntToStr(N)+':00');
              end;
            end;
          Inc(N);
          X := X + Sections;
         end;
     with ClientRect do
      begin
       Pen.Style := psSolid;
       Brush.Color := FDateBoxColor;
       NewRect := ClientRect;
       NewRect.Top := 2;
       NewRect.Bottom := RowHeight;
       NewRect.Left := 3;
       NewRect.Right := Width;  {  Width - 2 }
       FillRect(NewRect);
       SavedColor := Font.Color;
       Font.Assign(FDateTextFont);
       if (FStartHour - FTimeSlots) < 0 then
        begin
         TextOut((Width div 2)-(TextWidth(FDateText) div 2), 4, FDateText);
        end
       else
        begin
         TextOut((Width div 2)-(Width div 4)-(TextWidth(FDateText) div 2), 4, FDateText);
        end;
       DrawButtons;
       Font.Color := SavedColor;
       Brush.Color := Color;
       Pen.Color := FShadowColor;
       MoveTo(1,RowHeight);
       LineTo(Width-2,RowHeight);
       Pen.Color := FLineColor;
       MoveTo(1,RowHeight-1);
       LineTo(Width-2,RowHeight-1);
       Pen.Color := FHighlightColor;
       MoveTo(Width-1,1);
       LineTo(Width-1,Height-1);
       MoveTo(Width-1,Height-1);
       LineTo(1,Height-1);
       Pen.Color := FShadowColor;
       MoveTo(0,0);
       LineTo(Width-1,0);
       MoveTo(1,1);
       LineTo(1,Height-1);
      end;
     DrawSelectedTimeSlots;
   end;
   if Bounding then
      begin
         Canvas.Pen.Mode := pmNot;
         Canvas.Pen.Style := psDot;
         Canvas.Brush.Style := FSelectBrush;
      end;
end;

procedure TTimeBar.RefreshSelect;
begin
 with Canvas do
  begin
   Brush.Color := FSelectedColor;
   FillRect(RefreshRect);
   DrawSegments;
  end;
end;

procedure TTimeBar.DrawSelect(DrawRect: TRect);
begin
 RefreshRect := DrawRect;
 with Canvas do
  begin
   Brush.Color := FSelectedColor;
   FillRect(RefreshRect);
  end;
end;

function PointInRect( const rectTest: TRect; X, Y: integer ): boolean;
begin
  Result := ( ( X >= rectTest.Left ) and ( X <= rectTest.Right ) and
     ( Y >= rectTest.Top ) and ( Y <= rectTest.Bottom ) );
end;

(*********************************************
Determine which token the user is on and highlight
the entire text of that token.
*********************************************)
procedure TTimeBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 CheckRow: Integer;
begin
  {Check if mouse was pressed in Left button area}
  if PointInRect(GetLeftButtonRect, X, Y) then
     begin
       PrevHour;
       Refresh;
     end;

  {Check if mouse was pressed in Right button area}
  if PointInRect(GetRightButtonRect, X, Y) then
     begin
       NextHour;
       Refresh;
     end;
  AnchorX := X; CurX := X;
  for CheckRow := 2 to FNumberOfRows+2 do
   begin
    if (Y >= RowHeight*CheckRow) and (Y < RowHeight*(CheckRow+1)) then
     begin
      AnchorY := (RowHeight*CheckRow)+3;
      CurY := (RowHeight*(CheckRow+1));
      if (Y > (RowHeight*CheckRow)) and ((X > 1) or (X < Width-1)) then Bounding := true;
     end;
   end;
  inherited MouseDown( Button, Shift, X, Y );
end;

(*********************************************
Determine which token the user is on and highlight
the entire text of that token.
*********************************************)
procedure TTimeBar.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: integer );
var
 SelectedRect: TRect;
 Sections,
 CheckRow,
 N: Integer;
begin
  if Bounding then
   with Canvas do
    begin
     Bounding := false;
     Pen.Style := psDot;
     Pen.Mode := pmNot;
     Brush.Style := FSelectBrush;
     Rectangle(AnchorX, AnchorY, CurX, CurY);
     Pen.Mode := pmCopy;
     Brush.Style := bsSolid;
     if ([ssCtrl] <= Shift) then
      begin
      end
     else
      begin
       Brush.Color := Color;
       SelectedRect.Top := AnchorY;
       SelectedRect.Left := 2;
       SelectedRect.Bottom := CurY;
       SelectedRect.Right := Width; { Was Width - 2 }
       FillRect(SelectedRect);
      end;
      Sections := (Width-4) div FTimeSlots;
      N := FStartHour;
      X := 2;
      while X < Width-1 do
        begin
         Pen.Color := FLineSelectColor;
         Pen.Style := psSolid;
         MoveTo(X,RowHeight*2);
         LineTo(X, Height-2);
         if not FHalfHour then
          begin
           MoveTo(X+(Sections div 4),RowHeight*2);
           LineTo(X+(Sections div 4), Height-2);
          end;
         MoveTo(X+(Sections div 2),RowHeight*2);
         LineTo(X+(Sections div 2), Height-2);
         if not FHalfHour then
          begin
           MoveTo(X+((Sections div 2)+(Sections div 4)),RowHeight*2);
           LineTo(X+((Sections div 2)+(Sections div 4)), Height-2);
          end;

         Brush.Color := FSelectedColor;
         SelectedRect.Top := AnchorY;
         SelectedRect.Bottom := CurY;

         if FHalfHour then
          begin
           if (AnchorX >= X) and (AnchorX < X+(Sections div 2)) then AnchorX := X;
           if (AnchorX >= X+(Sections div 2)) and (AnchorX < X+Sections) then
               AnchorX := X+(Sections div 2);

           if (CurX >= X) and (CurX < X+(Sections div 2)) then CurX := X;
           if (CurX >= X+(Sections div 2)) and (CurX < X+Sections) then
               CurX := X+(Sections div 2);
          end
         else
          begin
           if (AnchorX >= X) and (AnchorX < X+(Sections div 4)) then AnchorX := X;
           if (AnchorX >= X+(Sections div 4)) and (AnchorX < X+(Sections div 2)) then AnchorX := X+(Sections div 4);
           if (AnchorX >= X+(Sections div 2)) and (AnchorX < X+(Sections div 2)+(Sections div 4)) then
               AnchorX := X+(Sections div 2);
           if (AnchorX >= X+(Sections div 2)+(Sections div 4)) and (AnchorX < X+Sections) then
               AnchorX := X+(Sections div 2)+(Sections div 4);

           if (CurX >= X) and (CurX < X+(Sections div 4)) then CurX := X;
           if (CurX >= X+(Sections div 4)) and (CurX < X+(Sections div 2)) then CurX := X+(Sections div 4);
           if (CurX >= X+(Sections div 2)) and (CurX < X+(Sections div 2)+(Sections div 4)) then
               CurX := X+(Sections div 2);
           if (CurX >= X+(Sections div 2)+(Sections div 4)) and (CurX < X+Sections) then
                CurX := X+(Sections div 2)+(Sections div 4);
          end;

         if CurX > AnchorX then
          begin
           SelectedRect.Left := AnchorX;
           SelectedRect.Right := CurX;
          end
         else
          begin
           SelectedRect.Left := CurX;
           SelectedRect.Right := AnchorX;
          end;

         Inc(N);
         X := X + Sections;
        end;
     if CurX > AnchorX then
      begin
       if FHalfHour then
        begin
         CurX := CurX + (Sections div 2);
        end
       else
        begin
         CurX := CurX + (Sections div 4);
        end;
       SelectedRect.Left := AnchorX;
       SelectedRect.Right := CurX;
      end
     else
      begin
       if FHalfHour then
        begin
         AnchorX := AnchorX + (Sections div 2);
        end
       else
        begin
         AnchorX := AnchorX + (Sections div 4);
        end;
       SelectedRect.Left := CurX;
       SelectedRect.Right := AnchorX;
      end;
     DrawSelect(SelectedRect);
     if ([ssCtrl] <= Shift) then
      begin
       for CheckRow := 2 to FNumberOfRows+2 do
        begin
         if (AnchorY >= RowHeight*CheckRow) and (AnchorY < RowHeight*(CheckRow+1)) then
          begin
           FTimeListStart[CheckRow-1].Add(GetTimeSlotA(SelectedRect.Left div (Sections div 4)));
           if FFormatTime = TRUE then
              begin
                 if FAMPM then
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '12:15 AM' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('11:59 PM');
                  end
                 else
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '00:15' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('23:59');
                  end;
              end
           else
              begin
                 if FAMPM then
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '12:15 AM' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('11:59 PM');
                  end
                 else
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '0015' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('2359');
                  end;
              end;
          end;
        end;
      end
     else
      begin
       for CheckRow := 2 to FNumberOfRows+2 do
        begin
         if (AnchorY >= RowHeight*CheckRow) and (AnchorY < RowHeight*(CheckRow+1)) then
          begin
           FTimeListStart[CheckRow-1].Clear;
           FTimeListEnd[CheckRow-1].Clear;
           FTimeListStart[CheckRow-1].Add(GetTimeSlotA(SelectedRect.Left div (Sections div 4)));
           if FFormatTime = TRUE then
              begin
                 if FAMPM then
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '12:15 AM' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('11:59 PM');
                  end
                 else
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '00:15' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('23:59');
                  end;
              end
           else
              begin
                 if FAMPM then
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '12:15 AM' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('11:59 PM');
                  end
                 else
                  begin
                   if GetTimeSlotA(SelectedRect.Right div (Sections div 4)) >= '0015' then
                      FTimeListEnd[CheckRow-1].Add(GetTimeSlotB(SelectedRect.Right div (Sections div 4)))
                   else
                      FTimeListEnd[CheckRow-1].Add('2359');
                  end;
              end;
          end;
        end;
      end;
     DrawSegments;
   end;
  if Assigned(FOnTimeSelected) then OnTimeSelected(Self);
  inherited MouseUp( Button, Shift, X, Y );
end;

(*********************************************
Determine which token the user is on and highlight
the entire text of that token.
*********************************************)
procedure TTimeBar.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Bounding then
  with Canvas do
   begin
    Pen.Mode := pmNot;
    Pen.Width := 1;
    Pen.Style := psDot;
    Brush.Style := FSelectBrush;
    Rectangle(AnchorX, AnchorY, CurX, CurY);
    CurX := X;
    if CurX < 2 then CurX := 2;
    if CurX > Width-4 then CurX := Width-4;
    Rectangle(AnchorX, AnchorY, CurX, CurY);
   end;
  inherited MouseMove( Shift, X, Y );
end;

procedure TTimeBar.DrawSegments;
var
 Sections, N, X : Word;
 DrawRow: Integer;
begin
 with Canvas do
  begin
   Sections := (Width-4) div FTimeSlots;
   N := FStartHour; X := 2;
   for DrawRow := 2 to FNumberOfRows+2 do
    begin
     if DrawRow > 2 then
     begin
      Pen.Color := Color;
      MoveTo(3,(RowHeight*DrawRow));
      LineTo(Width-3,(RowHeight*DrawRow));
     end;
     if DrawRow < FNumberOfRows+2 then
     begin
      Pen.Color := Color;
      MoveTo(3,(RowHeight*DrawRow)+2);
      LineTo(Width-3,(RowHeight*DrawRow)+2);
     end;
    end;
    while X < Width-1 do
     begin
       Pen.Mode := pmCopy;
       Pen.Style := psSolid;
       Pen.Color := Color;
       Pen.Width := 3;
       MoveTo(X,(RowHeight*2)+3);
       LineTo(X, Height-6);
       if not FHalfHour then
        begin
         MoveTo(X+(Sections div 4),(RowHeight*2)+3);
         LineTo(X+(Sections div 4), Height-6);
        end;
       MoveTo(X+(Sections div 2),(RowHeight*2)+3);
       LineTo(X+(Sections div 2), Height-6);
       if not FHalfHour then
        begin
         MoveTo(X+((Sections div 2)+(Sections div 4)),(RowHeight*2)+3);
         LineTo(X+((Sections div 2)+(Sections div 4)), Height-6);
        end;
       Pen.Width := 1;
       Pen.Color := FLineSelectColor;
       MoveTo(X,(RowHeight*2)+3);
       LineTo(X, Height-3);
       if not FHalfHour then
        begin
         MoveTo(X+(Sections div 4),(RowHeight*2)+3);
         LineTo(X+(Sections div 4), Height-3);
        end;
       MoveTo(X+(Sections div 2),(RowHeight*2)+3);
       LineTo(X+(Sections div 2), Height-3);
       if not FHalfHour then
        begin
         MoveTo(X+((Sections div 2)+(Sections div 4)),(RowHeight*2)+3);
         LineTo(X+((Sections div 2)+(Sections div 4)), Height-3);
        end;
       Inc(N);
       X := X + Sections;
     end;
    Pen.Color := FLineColor;
    MoveTo(2,(RowHeight*2));
    LineTo(2, Height-1);
    MoveTo(Width-2,(RowHeight*2));
    LineTo(Width-2, Height-1);
    Pen.Color := FShadowColor;
    MoveTo(1,(RowHeight*2));
    LineTo(1, Height-1);
    Pen.Color := FHighlightColor;
    MoveTo(Width-1,(RowHeight*2));
    LineTo(Width-1, Height-1);
    for DrawRow := 2 to FNumberOfRows+2 do
     begin
      Pen.Color := FLineColor;
      MoveTo(3,(RowHeight*DrawRow)+1);
      LineTo(Width-3,(RowHeight*DrawRow)+1);
     end;
   end;
end;

procedure TTimeBar.SetTimeBarColor(Value: TColor);
begin
  if FBackgroundColor <> Value then
   begin
     FBackgroundColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetLineColor(Value: TColor);
begin
  if FLineColor <> Value then
   begin
     FLineColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetShadowColor(Value: TColor);
begin
  if FShadowColor <> Value then
   begin
     FShadowColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
   begin
     FHighlightColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetDateBoxColor(Value: TColor);
begin
  if FDateBoxColor <> Value then
   begin
     FDateBoxColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetLineSelectColor(Value: TColor);
begin
  if FLineSelectColor <> Value then
   begin
     FLineSelectColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetSelectedColor(Value: TColor);
begin
  if FSelectedColor <> Value then
   begin
     FSelectedColor := Value;
     Invalidate;
   end;
end;

procedure TTimeBar.SetTimeSlots(Value: Integer);
begin
 if FTimeSlots <> Value then
  begin
   if Value < 1 then
    begin
     FTimeSlots := 1;
    end
   else
   if (Value > 24) then
    begin
     FTimeSlots := 24;
    end
   else
    begin
     FTimeSlots := Value;
    end;
   Invalidate;
  end;
end;

procedure TTimeBar.SetStartHour(Value: Integer);
begin
 if FStartHour <> Value then
  begin
   if FAMPM then
    begin
     if (Value > 23) or (Value < 0) then
      begin
       FStartHour := 0;
      end
     else
      begin
       FStartHour := Value;
      end;
    end
   else
    begin
     if (Value > 23) or (Value < 0) then
      begin
       FStartHour := 0;
      end
     else
      begin
       FStartHour := Value;
      end;
    end;
   Invalidate;
  end;
end;

procedure TTimeBar.SetNumberOfRows(Value: Integer);
begin
 if FNumberOfRows <> Value then
  begin
   if (Value >= 1) and (Value <= MAXNUMBEROFROWS) then
    FNumberOfRows := Value
   else
    FNumberOfRows := 1;
  end;
 Invalidate;
end;

function TTimeBar.GetTimeSlotA(Value: Integer): String;
var
 OldTime,
 TempTime,
 NewTime: TDateTime;
begin
 OldTime := EncodeTime(FStartHour,0,0,0);
 NewTime := EncodeTime(FStartHour,15,0,0);
 TempTime := NewTime - OldTime;
 if Value > 0 then OldTime := OldTime + (Value * TempTime);
 if FFormatTime = FALSE then Result := IntToStr(StrToIntDef(FormatDateTime('hhmm', OldTime),0))
 else
    if AMPMFormat = TRUE then Result := FormatDateTime('hh:mm AM/PM', OldTime)
 else
    Result := FormatDateTime('hh:mm', OldTime);
end;

function TTimeBar.GetTimeSlotB(Value: Integer): String;
var
 OldTime,
 TempTime,
 NewTime: TDateTime;
begin
 OldTime := EncodeTime(FStartHour,0,0,0);
 NewTime := EncodeTime(FStartHour,15,0,0);
 TempTime := NewTime - OldTime;
 if Value > 0 then OldTime := OldTime + (Value * TempTime);
 OldTime := OldTime - EncodeTime(0,1,0,0);
 if FFormatTime = FALSE then Result := IntToStr(StrToIntDef(FormatDateTime('hhmm', OldTime),0))
 else
    if AMPMFormat = TRUE then Result := FormatDateTime('hh:mm AM/PM', OldTime)
 else
    Result := FormatDateTime('hh:mm', OldTime);
end;

function TTimeBar.GetSlotNumber(Value: String): Integer;
var
 OldTime,
 ConvertTime,
 NewTime: TDateTime;
 Hour,
 Minute,
 Second,
 MSec: Word;
 TimeSlot: Integer;
begin
  begin
   try
   OldTime := EncodeTime(FStartHour,0,0,0);
   NewTime := EncodeTime(FStartHour,15,0,0);
    try
     ConvertTime := StrToTime(FormatAsTime(Value));
    except
     ConvertTime := EncodeTime(FStartHour,0,0,0);
    end;
   TimeSlot := 0;
   while ((ConvertTime - OldTime) > 0) do
    begin
     Inc(TimeSlot);
     DecodeTime(OldTime,Hour,Minute,Second,MSec);
     if Minute < 45 then Minute := Minute + 15 else
      begin
       Hour := Hour + 1;
       Minute := 0;
      end;
     if Hour = 24 then
        begin
           Hour := 23;
           Minute := 59;
        end;
     OldTime := EncodeTime(Hour,Minute,Second,MSec);
    end;
   Result := TimeSlot;
   except
    begin
     Result := 0;
    end;
   end;
  end;
end;

function TTimeBar.FormatAsTime(Value: String): String;
begin
   Result := '0:00';
   while Length(Value) < 3 do Value := '0'+Value;
   if (Result <> '') then
      Result := Copy(Value, 1, Length(Value)-2) + ':' + Copy(Value, Length(Value)-1, Length(Value));
   try
      StrToTime(Result);
   except
      Result := '0:00';
   end;
end;

procedure TTimeBar.SetTimeSlotIndex(Value: Integer);
begin
 if FTimeSlotIndex <> Value then
  begin
   if (Value >= 1) and (Value <= MAXNUMBEROFROWS) then
    FTimeSlotIndex := Value
   else
    FTimeSlotIndex := 1;
  end;
end;

procedure TTimeBar.SetAMPM(Value: Boolean);
begin
 if FAMPM <> Value then
  begin
   FAMPM := Value;
   if (FAMPM = TRUE) and (FStartHour > 12) then SetStartHour(FStartHour-12);
   Invalidate;
  end;
end;

procedure TTimeBar.SetHalfHour(Value: Boolean);
begin
 if FHalfHour <> Value then
  begin
   FHalfHour := Value;
   Invalidate;
  end;
end;

procedure TTimeBar.SetDateText(Value: String);
var
 TempDate: TDateTime;
begin
 if FDateText <> Value then
  begin
   FDateText := Value;
   Invalidate;
  end;
end;

procedure TTimeBar.SetCenterTime(Value: Boolean);
begin
 if FCenterTime <> Value then
  begin
   FCenterTime := Value;
   Invalidate;
  end;
end;

procedure TTimeBar.SetDateTextFont(Value: TFont);
begin
  if FDateTextFont <> Value then
   begin
     FDateTextFont.Assign( Value );
     Invalidate;
   end;
end;

procedure TTimeBar.SetAbout(Value: TAbout);
begin
  if FAbout <> Value then
     FAbout.Assign( Value );
     {Create default greeting list}
     MessageBeep(0);
     FAbout.Clear;
     FAbout.Add('TTimeBar - Version 1.1');
     FAbout.Add('-----------------------------');
     FAbout.Add(' ');
     FAbout.Add('No other compares to Delphi!');
     FAbout.Add(' ');
     FAbout.Add('Copyright  1996 by Jim Wood');
end;

function  TTimeBar.GetTimeSlotsStart: TTimeSlots;
begin
 Result := FTimeListStart[FTimeSlotIndex];
end;

function  TTimeBar.GetTimeSlotsEnd: TTimeSlots;
begin
 Result := FTimeListEnd[FTimeSlotIndex];
end;

procedure TTimeBar.SetTimeSlotsStart(Value: TTimeSlots);
begin
  if FTimeListStart[FTimeSlotIndex] <> Value then
   begin
    FTimeListStart[FTimeSlotIndex].Assign( Value );
    Invalidate;
   end;
end;

procedure TTimeBar.SetTimeSlotsEnd(Value: TTimeSlots);
begin
  if FTimeListEnd[FTimeSlotIndex] <> Value then
   begin
    FTimeListEnd[FTimeSlotIndex].Assign( Value );
    Invalidate;
   end;
end;

procedure TTimeBar.DrawSelectedTimeSlots;
var
 SelectedRect: TRect;
 SlotStart,
 SlotEnd,
 RowCheck,
 SlotID: Integer;
begin
 for RowCheck := 2 to FNumberOfRows+2 do
  begin
   if FTimeListStart[RowCheck-1].Count <> FTimeListEnd[RowCheck-1].Count then Exit;
   if not Bounding then
      begin
         AnchorY := (RowHeight*RowCheck)+3;
         CurY := (RowHeight*(RowCheck+1));
      end;
         SelectedRect.Top := AnchorY;
         SelectedRect.Bottom := CurY;
   for SlotID := 0 to FTimeListStart[RowCheck-1].Count - 1 do
    begin
     SlotStart := GetSlotNumber(FTimeListStart[RowCheck-1][SlotID]);
     SlotEnd := GetSlotNumber(FTimeListEnd[RowCheck-1][SlotID]);
     if SlotStart > SlotEnd then
      begin
       SlotEnd := (Width-4) div (((Width-4) div FTimeSlots)div 4);
      end;
      if not Bounding then
      begin
         SelectedRect.Left := (SlotStart*((Width-4) div FTimeSlots)div 4)+2;
         SelectedRect.Right := (SlotEnd*((Width-4) div FTimeSlots)div 4)+2;
      end
     else
      begin
         SelectedRect.Left := CurX;
         SelectedRect.Right := CurY;
      end;
     DrawSelect(SelectedRect);
    end;
  end;
 DrawSegments;
end;

destructor TTimeBar.Destroy;
var
 I: Integer;
begin
  FAbout.Free;
  for I := 1 to MAXNUMBEROFROWS do
   begin
    FTimeListStart[I].Free;
    FTimeListEnd[I].Free;
   end;
  FDateTextFont.Free;
  inherited Destroy;
end;
end.

